mod elastic_conection;
mod nut;
use nut::nut_client::NutClient;
use elastic_conection::ElasticConection;
use chrono;
use tokio::time::{sleep, Duration};
use std::fmt;

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    //println!("{}",chrono::offset::Utc::now().to_rfc3339_opts(chrono::SecondsFormat::Secs, true));
    env_logger::Builder::from_default_env()
        .format_timestamp_secs()
        .filter(None, log::LevelFilter::Info)
        .init();
    let nut = NutClient::new().unwrap();
    match ElasticConection::new() {
        Ok(c) => start_loop(c, nut).await,
        Err(e) => println!("{}",e),
    }
    Ok(())
}

async fn start_loop(e_con: ElasticConection, mut n_con: NutClient) {
    let mut has_failed = false;
    loop {
        sleep(Duration::from_secs(10)).await;
        let payload = match n_con.get_data(chrono::offset::Utc::now().to_rfc3339_opts(chrono::SecondsFormat::Secs, true)) {
            Ok(p) => p,
            Err(e) => {
                    process_error(Box::new(e), "nut connection", &mut has_failed);
                    continue;
                },
        };
        match e_con.send(payload).await {
            Ok(()) => has_failed = false,
            Err(e) => {
                    process_error(Box::new(e), "elasticsearch", &mut has_failed);
                    continue;
                },
        }
    }
}

fn process_error(e: Box<dyn fmt::Display>, system: &str, has_failed: &mut bool) {
    if !*has_failed {
        *has_failed = true;
        log::error!("error in {}: {}", system, e);
    }
}
