use nut_client::blocking::Connection;
use nut_client::ConfigBuilder;
use serde_json::{json, Value};
use serde::{Deserialize, Serialize};
use std::collections::HashMap;

pub struct NutClient {
    conection: Connection,
}

#[derive(Serialize, Deserialize)]
struct Ups {
    name: String,
    description: String,
    variables: HashMap<String, DataTypes>,
}

#[derive(Serialize, Deserialize)]
enum DataTypes {
    Num(f64),
    Str(String),
}

impl NutClient {
    pub fn new() -> Result<Self, Box<dyn std::error::Error>> {
        let config = ConfigBuilder::new()
                .with_host(("10.0.0.1".to_string().into(), 3493 as u16).try_into().unwrap_or_default())
                .with_debug(false) // Turn this on for debugging network chatter
                .build();
        Ok(Self{
            conection: Connection::new(&config)?,
        })
    }

    pub fn get_data(&mut self, date: String) -> Result<Value, Box<dyn std::error::Error>> {
        let mut ret = Vec::new();
        for (nname, ndescription) in self.conection.list_ups()? {
            let mut data : HashMap<String, DataTypes> = HashMap::new();
            for var in self.conection.list_vars(&nname)? {
                let mut svar = var.to_string();
                let separator = svar.find(":").unwrap();
                let value = svar.split_off(separator);
                svar = svar.replace(".", "_");
                let usable_value = value.strip_prefix(": ").unwrap();
                match usable_value.parse::<i64>() {
                    Ok(num) => data.insert(svar,DataTypes::Num(num as f64)),
                    Err(_e) => data.insert(svar,DataTypes::Str(usable_value.to_string())),
                };
                //data.insert(svar, value.strip_prefix(": ").unwrap().to_string());
            }
            ret.push(Ups {
                name: nname,
                description: ndescription,
                variables: data,
            });
        }
        Ok(
            json!({
                "@timestamp" : date,
                "UPS" : ret,
            })
        )
    }
}
